import os
import requests

from rmbclient.exceptions import ResourceExists, Unauthorized, ResourceNotFound, ServerInternalError


class APIRequest:
    ## TODO: 添加异常处理

    token = os.environ.get("RMB_TOKEN", "token1")
    api_url = os.environ.get("RMB_API_URL", "http://127.0.0.1:5000")
    debug = os.environ.get("RMB_DEBUG", "True").lower() == "true"

    def _handle_response(self, response, method, endpoint, data):
        if self.debug:
            print(f"Request: {method} {endpoint} with Data {data}")
            print(f"Response: {response.status_code} {response.text}")
        return response.json()


    def send(self, endpoint, method, data=None) -> dict or None:
        headers = {"Authorization": f"Bearer {self.token}"}
        url = f"{self.api_url}{endpoint}"
        if method == "GET":
            response = requests.get(url, headers=headers)
        elif method == "POST":
            response = requests.post(url, json=data, headers=headers)
        elif method == "DELETE":
            response = requests.delete(url, headers=headers)
        else:
            raise ValueError("Unsupported HTTP method")

        if response.status_code in (200, 201):
            if method == 'DELETE':
                return True
            else:
                return response.json()

        try:
            _res = response.json()
        except Exception:
            _res = str(response)

        log_record = {'response': _res, 'method': method, 'endpoint': endpoint, 'data': data}

        if response.status_code == 409:
            raise ResourceExists("Resource already exists")
        elif response.status_code == 401:
            raise Unauthorized("Unauthorized")
        elif response.status_code in (404, 403):
            raise ResourceNotFound(f"Resource not found: {log_record}")
        elif response.status_code == 500:
            raise ServerInternalError(f"Server Internal Error: {log_record}")
        elif response.status_code == 400:
            raise ValueError(f"Bad Request: {log_record}")
        else:
            raise Exception(f"Error: {log_record}")


rmb_api = APIRequest()


from rmbcommon.models import ChatCore, BIAnswer
from rmbclient.models.base import convert_to_object, BaseResourceList
from rmbclient.models.run import RunList
from rmbclient.models.message import MessageList
from rmbclient.api import rmb_api


class ChatClientModel(ChatCore):

    def delete(self):
        return rmb_api.send(endpoint=f"/chats/{self.id}", method="DELETE")

    @property
    def runs(self):
        return RunList(endpoint=f"/chats/{self.id}/runs")

    @property
    def messages(self):
        return MessageList(endpoint=f"/chats/{self.id}/messages")

    @convert_to_object(cls=BIAnswer)
    def ask(self, question):
        # 提问
        data = {"question": question}
        return rmb_api.send(endpoint=f"/chats/{self.id}", method="POST", data=data)


class ChatList(BaseResourceList):
    @convert_to_object(cls=ChatClientModel)
    def _get_all_resources(self):
        # 获取所有资源
        return rmb_api.send(endpoint=self.endpoint, method="GET")

    @convert_to_object(cls=ChatClientModel)
    def get(self, id):
        # 通过资源ID来获取
        return rmb_api.send(endpoint=f"{self.endpoint}{id}", method="GET")

    @convert_to_object(cls=ChatClientModel)
    def create(self, datasource_ids):
        data = {"datasource_ids": datasource_ids}
        return rmb_api.send(endpoint=self.endpoint, method="POST", data=data)




from rmbclient.models import DataResourceList, ChatList
from rmbclient.api import rmb_api


class ReliableMetaBrain:

    def __init__(self, api_url='http://127.0.0.1:5000',
                 token='token1', debug=False):
        rmb_api.api_url = api_url
        rmb_api.token = token
        rmb_api.debug = debug

    @property
    def datasources(self):
        return DataResourceList(endpoint="/datasources/")

    @property
    def chats(self):
        return ChatList(endpoint="/chats/")

    def test_clear_all(self):
        return rmb_api.send(endpoint="/tests/clear_data/all", method="POST")

    def test_clear_brain(self):
        return rmb_api.send(endpoint="/tests/clear_data/brain", method="POST")

    def test_clear_chat(self):
        return rmb_api.send(endpoint="/tests/clear_data/chat", method="POST")




import argparse

def main():
    # 设置命令行参数解析
    parser = argparse.ArgumentParser(description="Interactive CLI for RMB client.")
    parser.add_argument('-a', '--api_url', type=str, required=True, help='The API URL for the RMB client.')
    parser.add_argument('-t', '--token', type=str, required=True, help='The token for authentication.')

    args = parser.parse_args()

    # 导入 RMB 类
    from rmbclient import RMB
    rmb = RMB(token=args.token, api_url=args.api_url)
    print("\n----- RMB 客户端初始化完成！您可以使用 'rmb' 来访问RMB的方法。-----\n")

    try:
        # 尝试导入 IPython
        from IPython import start_ipython
        start_ipython(argv=[], user_ns={'rmb': rmb})
    except ImportError:
        # 如果 IPython 未安装，使用标准 Python shell
        import code
        code.interact(local=locals())


if __name__ == "__main__":
    main()




class BaseCoreModel:
    __init_dict_keys__ = []

    def __init__(self, **kwargs):
        for k in self.__init_dict_keys__:
            setattr(self, k, kwargs.get(k))

    @classmethod
    def load_from_dict(cls, data: dict):
        # log.debug(f"Loading from dict: {data}")
        if not data:
            return None
        return cls(**data)

    def to_dict(self):
        data = {}
        for k in self.__init_dict_keys__:
            data[k] = getattr(self, k)
        return data
